<?php
function katerina_helper_init_gallery_block() {

	if ( ! function_exists( 'register_block_type' ) ) {
		return;
	}

	wp_register_script( 'katerina-gallery-editor', KATERINA_HELPER_PLUGIN_URL . 'blocks/gallery/index.js', array( 'wp-blocks', 'wp-element' ), filemtime( KATERINA_HELPER_PLUGIN_PATH . 'blocks/gallery/index.js' ) );

	wp_register_style( 'katerina-gallery-editor', KATERINA_HELPER_PLUGIN_URL . 'blocks/gallery/style-editor.css', array( 'wp-edit-blocks' ) );

	$render_callback = apply_filters( 'katerina_helper_gallery_render_callback', '' );

	register_block_type( 'katerina/gallery', array(
		'editor_style'    => 'katerina-gallery-editor',
		'editor_script'   => 'katerina-gallery-editor',
		'render_callback' => $render_callback
	) );

	if ( $render_callback ) {
		add_shortcode( 'katerina-gallery', $render_callback );
	}
}
add_action( 'init', 'katerina_helper_init_gallery_block' );


// Allow iframes and video tags in the featured area (the safe way)
function katerina_helper_gallery_duotone_class( $attributes ) {
	if ( isset( $attributes['style']['color']['duotone'] ) && is_array( $attributes['style'] ) && function_exists( 'wp_render_duotone_support' ) ) {
		// track development changes in /wp-includes/block-supports/duotone.php - wp_render_duotone_support
		$preset = array(
			'slug'   => wp_unique_id( sanitize_key( implode( '-', $attributes['style']['color']['duotone'] ) . '-' ) ),
			'colors' => $attributes['style']['color']['duotone'],
		);
		$filter_property = wp_get_duotone_filter_property( $preset );
		$filter_id       = wp_get_duotone_filter_id( $preset );
		$filter_svg      = wp_get_duotone_filter_svg( $preset );

		$scope     = '.' . $filter_id;
		$selectors = explode( ',', '.katerina-gallery-entry > a > img' );
		$scoped    = array();
		foreach ( $selectors as $sel ) {
			$scoped[] = $scope . ' ' . trim( $sel );
		}
		$selector = implode( ', ', $scoped );

		$filter_style = $selector . ' { filter: ' . $filter_property . '; }';

		wp_register_style( $filter_id, false, array(), true, true );
		wp_add_inline_style( $filter_id, $filter_style );
		wp_enqueue_style( $filter_id );

		add_action(
			'wp_footer',
			static function () use ( $filter_svg, $selector ) {
				echo $filter_svg;

				/* Safari hack to force repaint for duotone SVG filters */
				global $is_safari;
				if ( $is_safari ) {
					printf(
						'<script>( function() { var el = document.querySelector( %s ); var display = el.style.display; el.style.display = "none"; el.offsetHeight; el.style.display = display; } )();</script>',
						wp_json_encode( $selector )
					);
				}
			}
		);

		return ' ' . $filter_id;
	}
	return '';
}
add_filter( 'katerina_gallery_additional_classes', 'katerina_helper_gallery_duotone_class', 10, 1 );
