( function ( __, element ) {
	var el = element.createElement;

	window.wp.blocks.registerBlockType( 'katerina/gallery', {
		title: __( 'Portfolio Gallery' ),
		description: __( 'Dynamic post gallery with custom layout and filters.' ),
		icon: el(
			'svg', { width: 24, height: 24, role: 'img', focusable: 'false' },
			el('path', { d: 'M21,13h-6v8h6M3,13v8h10v-8M3,3v8h6V3m2,0v8h10V3Z' })
		),
		category: 'media',
		keywords: [ __('portfolio'), __( 'gallery' ) ],
		supports: {
			html: false,
			align: true,
			color: {
				__experimentalDuotone: '.gallery-entry > .gallery-entry-image, .katerina-gallery-entry > a > img'
			}
		},
		attributes: {
			layout: {
				type: 'string',
				default: 'dashboard',
			},
			switch_view: {
				type: 'boolean',
				default: false,
			},
			grow_last: {
				type: 'boolean',
				default: false,
			},
			filters: {
				type: 'boolean',
				default: false,
			},
			filter_style: {
				type: 'string',
				default: 'opacity',
			},
			post_type: {
				type: 'string',
				default: 'post',
			},
			cat: {
				type: 'number',
				default: -1,
			},
			post_type: {
				type: 'string',
				default: 'post',
			},
			order: {
				type: 'string',
				default: 'DESC',
			},
			orderby: {
				type: 'string',
				default: 'menu_order date',
			},
			aspect_ratio: {
				type: 'number',
				default: 0,
			},
			caption: {
				type: 'boolean',
				default: true,
			},
			caption_style: {
				type: 'string',
				default: 'overlay-minimal',
			},
			caption_position: {
				type: 'string',
				default: 'top',
			},
			caption_theme: {
				type: 'string',
				default: 'normal',
			}
		},

		edit: function (props) {
			var attributes = props.attributes,
				setAttributes = props.setAttributes,
				categories_empty = [{ id: -1, name: ' ' }],
				categories = window.wp.data.useSelect( function ( select ) {
					return select( 'core' ).getEntityRecords( 'taxonomy', 'category', { per_page: -1, hide_empty: true, _fields: 'id,name' } );
				}, [] ),
				post_types = window.wp.data.useSelect( function ( select ) {
					return select( 'core' ).getPostTypes();
				}, [] );

			if ( post_types ) {
				post_types = post_types.filter( function( category ) {
					return ( true === category.visibility.show_in_nav_menus && true === category.visibility.show_ui && "page" !== category.slug );
				});
			}

			return [
				el(wp.blockEditor.InspectorControls, {
						key: 'inspector'
					},
					el(wp.components.PanelBody, {
							title: __('Gallery Settings'),
							initialOpen: true,
						},
						categories && categories.length > 0 && el(wp.components.SelectControl, {
							label: __('Category'),
							help: __('Determine the category from which the items are pulled (and from which the filters, as sub-categories, will be displayed).'),
							value: attributes.cat,
							options: categories_empty.concat(categories).map(({id, name}) => ({ value: id, label: name })),
							onChange: function(newVal) {
								setAttributes({ cat: parseInt(newVal) });
							},
						}),
						post_types && post_types.length > 0 && el(wp.components.SelectControl, {
							label: __('Post type'),
							help: __('You can use any defined custom post type (if you want separation from regular posts).'),
							value: attributes.post_type,
							options: post_types.map(({slug, name}) => ({ value: slug, label: name })),
							onChange: function(newVal) {
								setAttributes({ post_type: newVal });
							},
						}),
						el(wp.components.SelectControl, {
							label: __('Layout'),
							value: attributes.layout,
							options: [
								{ value: 'stream', label: __('Stream') },
								{ value: 'grid', label: __('Grid') },
								{ value: 'dashboard', label: __('Dashboard') }
							],
							onChange: function(newVal) {
								setAttributes({ layout: newVal });
							},
						}),
						el(wp.components.ToggleControl, {
							label: __('Switch between layouts'),
							checked: attributes.switch_view,
							onChange: function(newVal) {
								setAttributes({ switch_view: newVal });
							},
						}),
						attributes.layout && 'dashboard' === attributes.layout && el(wp.components.ToggleControl, {
							label: __('Grow last line'),
							checked: attributes.grow_last,
							onChange: function(newVal) {
								setAttributes({ grow_last: newVal });
							},
						}),
						el(wp.components.ToggleControl, {
							label: __('Display filters'),
							checked: attributes.filters,
							onChange: function(newVal) {
								setAttributes({ filters: newVal });
							},
						}),
						attributes.filters && el(wp.components.SelectControl, {
							label: __('Filter style'),
							value: attributes.filter_style,
							options: [
								{ value: 'hide', label: __('Hide completly') },
								{ value: 'opacity', label: __('Change opacity') },
							],
							onChange: function(newVal) {
								setAttributes({ filter_style: newVal });
							},
						}),
						el(wp.components.ToggleControl, {
							label: __('Display captions'),
							checked: attributes.caption,
							onChange: function(newVal) {
								setAttributes({ caption: newVal });
							},
						}),
						( 'stream' !== attributes.layout ) && attributes.caption && el(wp.components.SelectControl, {
							label: __('Caption style'),
							value: attributes.caption_style,
							options: [
								{ value: 'overlay-minimal', label: __('Overlay minimal') },
								{ value: 'overlay-excerpt', label: __('Overlay with excerpt') },
								{ value: 'panel-minimal', label: __('Panel minimal') },
								{ value: 'panel-excerpt', label: __('Panel with excerpts') }
							],
							onChange: function(newVal) {
								setAttributes({ caption_style: newVal });
							},
						}),
						( 'stream' !== attributes.layout ) && attributes.caption && 'overlay-excerpt' !== attributes.caption_style && el(wp.components.SelectControl, {
							label: __('Caption position'),
							value: attributes.caption_position,
							options: [
								{ value: 'top', label: __('Top') },
								{ value: 'bottom', label: __('Bottom') },
							],
							onChange: function(newVal) {
								setAttributes({ caption_position: newVal });
							},
						}),
						( 'stream' !== attributes.layout ) && attributes.caption && el(wp.components.SelectControl, {
							label: __('Caption theme'),
							value: attributes.caption_theme,
							options: [
								{ value: 'normal', label: __('Normal') },
								{ value: 'inverted', label: __('Inverted') },
							],
							onChange: function(newVal) {
								setAttributes({ caption_theme: newVal });
							},
						})
					)
				),
				el('div', window.wp.blockEditor.useBlockProps(),
					el('div', { className: 'katerina-gallery style-' + ( 'grid' !== attributes.layout ? attributes.layout : 'dashboard style-grid' ) + ( 'dashboard' === attributes.layout && attributes.grow_last ? ' grow-last-line' : '' ) + ( attributes.switch_view ? ' show-type-switch' : '' ) + ( ! attributes.filters ? ' no-filters' : '' ) },
						attributes.switch_view && el('div', { className: 'output-type' },
							el('span', { className: 'type-stream' + ( 'stream' === attributes.layout ? ' is-active' : '' ) },
								el( 'svg', { width: 24, height: 24, role: 'img', focusable: 'false' }, el('path', { d: 'M3,13v8h18v-8M3,3v8h18V3' }) )
							),
							el('span', { className: 'type-grid' + ( 'grid' === attributes.layout ? ' is-active' : '' ) },
								el( 'svg', { width: 24, height: 24, role: 'img', focusable: 'false' }, el('path', { d: 'M3,11H11V3H3M3,21H11V13H3M13,21H21V13H13M13,3V11H21V3' }) )
							),
							el('span', { className: 'type-dashboard' + ( 'dashboard' === attributes.layout ? ' is-active' : '' ) },
								el( 'svg', { width: 24, height: 24, role: 'img', focusable: 'false' }, el('path', { d: 'M21,13h-6v8h6M3,13v8h10v-8M3,3v8h6V3m2,0v8h10V3Z' }) )
							)
						),
						attributes.filters && el('div', { className: 'output-filter' },
							el('span', null, 'All'),
							el('span', null, 'Filter 1'),
							el('span', null, 'Filter 2'),
							el('span', null, 'Filter 3')
						),
						el('div', { className: 'gallery-output' },
							el('div', { className: 'gallery-entry' },
								el('span', { className: 'gallery-entry-image' }),
								attributes.caption && el('span', { className: 'gallery-entry-description style-' + attributes.caption_style + ' position-' + attributes.caption_position + ( 'inverted' === attributes.caption_theme ? ' invert-colors' : '' ) },
									el('strong', null, 'Caption sample'),
									( -1 === attributes.caption_style.indexOf( 'minimal' ) ) && el('span', null, 'Excerpt goes under the caption...'),
									( 'panel-minimal' !== attributes.caption_style ) && el('em', { className: 'gallery-entry-indicator'})
								)
							),
							el('div', { className: 'gallery-entry' }, el('span', { className: 'gallery-entry-image' }) ),
							el('div', { className: 'gallery-entry' }, el('span', { className: 'gallery-entry-image' }) ),
							( 'stream' !== attributes.layout ) && el('div', { className: 'gallery-entry' }, el('span', { className: 'gallery-entry-image' }) ),
							( 'stream' !== attributes.layout ) && el('div', { className: 'gallery-entry' }, el('span', { className: 'gallery-entry-image' }) ),
							( 'stream' !== attributes.layout ) && el('div', { className: 'gallery-entry' }, el('span', { className: 'gallery-entry-image' }) ),
							( 'dashboard' === attributes.layout ) && el('div', { className: 'gallery-entry' }, el('span', { className: 'gallery-entry-image' }) ),
						)
					)
				)
			];
		}
	} );
} )( wp.i18n.__, window.wp.element );
